@@ -9,6 +9,7 @@ from ._alias_selection import (
 )
 from ._data_format import DatasetDataFormatAccessor
 from ._downscale import DataArrayDownscalingAccessor, DatasetDownscalingAccessor
+from ._merge import DataArrayMergeAccessor, DatasetMergeAccessor
 from ._metadata import DatasetMetadataAccessor
 from ._overview import DataArrayOverviewAccessor, DatasetOverviewAccessor
 from ._setters import DataArraySettersAccessor, DatasetSettersAccessor
@@ -21,6 +22,7 @@ class PRIMAP2DatasetAccessor(
     DatasetAliasSelectionAccessor,
     DatasetDataFormatAccessor,
     DatasetDownscalingAccessor,
+    DatasetMergeAccessor,
     DatasetMetadataAccessor,
     DatasetOverviewAccessor,
     DatasetSettersAccessor,
@@ -34,6 +36,7 @@ class PRIMAP2DataArrayAccessor(
     DataArrayAggregationAccessor,
     DataArrayAliasSelectionAccessor,
     DataArrayDownscalingAccessor,
+    DataArrayMergeAccessor,
     DataArrayOverviewAccessor,
     DataArraySettersAccessor,
     DataArrayUnitAccessor,
