@@ -1,7 +1,13 @@
 """Data reading module of the PRIMAP2 climate policy analysis package."""
 
 
-from ._data_reading import read_long_csv_file_if, read_wide_csv_file_if
+from ._data_reading import (
+    convert_long_dataframe_if,
+    convert_wide_dataframe_if,
+    read_long_csv_file_if,
+    read_wide_csv_file_if,
+)
+from ._GHG_inventory_reading import nir_add_unit_information, nir_convert_df_to_long
 from ._interchange_format import (
     from_interchange_format,
     read_interchange_format,
@@ -11,7 +17,11 @@ from ._interchange_format import (
 __all__ = [
     "read_long_csv_file_if",
     "read_wide_csv_file_if",
+    "convert_long_dataframe_if",
+    "convert_wide_dataframe_if",
     "from_interchange_format",
     "read_interchange_format",
     "write_interchange_format",
+    "nir_convert_df_to_long",
+    "nir_add_unit_information",
 ]
