@@ -74,29 +74,27 @@ def test_downscale_timeseries(empty_ds):
     )
     expected = da.copy()
 
-    expected.loc[{"area (ISO3)": ["COL", "ARG", "MEX"], "source": "RAND2020"}] = (
-        np.broadcast_to(
-            np.concatenate(
-                [
-                    np.array([1, 1]),
-                    np.linspace(1 / 6, 2 / 8, 11) * np.array([6] * 9 + [8] * 2),
-                    np.linspace(2, 2 * 10 / 8, 8),
-                ]
-            ),
-            (3, 21),
-        ).T
-        * ureg("Gg CO2 / year")
-    )
-    expected.loc[{"area (ISO3)": "BOL", "source": "RAND2020"}] = (
+    expected.loc[
+        {"area (ISO3)": ["COL", "ARG", "MEX"], "source": "RAND2020"}
+    ] = np.broadcast_to(
         np.concatenate(
             [
-                np.array([3, 3]),
-                np.linspace(3 / 6, 2 / 8, 11) * np.array([6] * 9 + [8] * 2),
+                np.array([1, 1]),
+                np.linspace(1 / 6, 2 / 8, 11) * np.array([6] * 9 + [8] * 2),
                 np.linspace(2, 2 * 10 / 8, 8),
             ]
-        )
-        * ureg("Gg CO2 / year")
+        ),
+        (3, 21),
+    ).T * ureg(
+        "Gg CO2 / year"
     )
+    expected.loc[{"area (ISO3)": "BOL", "source": "RAND2020"}] = np.concatenate(
+        [
+            np.array([3, 3]),
+            np.linspace(3 / 6, 2 / 8, 11) * np.array([6] * 9 + [8] * 2),
+            np.linspace(2, 2 * 10 / 8, 8),
+        ]
+    ) * ureg("Gg CO2 / year")
 
     # we need a higher atol, because downscale_timeseries actually does the
     # downscaling using a proper calendar while here we use a calendar where all years
@@ -141,29 +139,27 @@ def test_downscale_timeseries(empty_ds):
 
     expected = da.copy()
 
-    expected.loc[{"area (ISO3)": ["COL", "ARG", "MEX"], "source": "RAND2020"}] = (
-        np.broadcast_to(
-            np.concatenate(
-                [
-                    np.array([1.2, 1.2, 1]),
-                    (np.linspace(1 / 5, 2 / 8, 11) * np.array([6] * 9 + [8] * 2))[1:],
-                    np.linspace(2, 2 * 10 / 8, 8),
-                ]
-            ),
-            (3, 21),
-        ).T
-        * ureg("Gg CO2 / year")
-    )
-    expected.loc[{"area (ISO3)": "BOL", "source": "RAND2020"}] = (
+    expected.loc[
+        {"area (ISO3)": ["COL", "ARG", "MEX"], "source": "RAND2020"}
+    ] = np.broadcast_to(
         np.concatenate(
             [
-                np.array([2.4, 2.4, 2]),
-                (np.linspace(2 / 5, 2 / 8, 11) * np.array([6] * 9 + [8] * 2))[1:],
+                np.array([1.2, 1.2, 1]),
+                (np.linspace(1 / 5, 2 / 8, 11) * np.array([6] * 9 + [8] * 2))[1:],
                 np.linspace(2, 2 * 10 / 8, 8),
             ]
-        )
-        * ureg("Gg CO2 / year")
+        ),
+        (3, 21),
+    ).T * ureg(
+        "Gg CO2 / year"
     )
+    expected.loc[{"area (ISO3)": "BOL", "source": "RAND2020"}] = np.concatenate(
+        [
+            np.array([2.4, 2.4, 2]),
+            (np.linspace(2 / 5, 2 / 8, 11) * np.array([6] * 9 + [8] * 2))[1:],
+            np.linspace(2, 2 * 10 / 8, 8),
+        ]
+    ) * ureg("Gg CO2 / year")
 
     assert_equal(downscaled, expected, equal_nan=True, atol=0.01)
 
@@ -178,33 +174,32 @@ def test_downscale_timeseries(empty_ds):
 
     expected.loc[
         {"area (ISO3)": ["COL", "ARG", "MEX", "BOL"], "source": "RAND2020"}
-    ] = (
-        np.broadcast_to(
-            np.concatenate(
-                [
-                    np.array(
-                        [
-                            np.nan,
-                            np.nan,
-                            1,
-                            np.nan,
-                            np.nan,
-                            6 / 4,
-                            6 / 4,
-                            6 / 4,
-                            6 / 4,
-                            6 / 4,
-                            6 / 4,
-                            2,
-                            2,
-                        ]
-                    ),
-                    np.linspace(2, 2 * 10 / 8, 8),
-                ]
-            ),
-            (4, 21),
-        ).T
-        * ureg("Gg CO2 / year")
+    ] = np.broadcast_to(
+        np.concatenate(
+            [
+                np.array(
+                    [
+                        np.nan,
+                        np.nan,
+                        1,
+                        np.nan,
+                        np.nan,
+                        6 / 4,
+                        6 / 4,
+                        6 / 4,
+                        6 / 4,
+                        6 / 4,
+                        6 / 4,
+                        2,
+                        2,
+                    ]
+                ),
+                np.linspace(2, 2 * 10 / 8, 8),
+            ]
+        ),
+        (4, 21),
+    ).T * ureg(
+        "Gg CO2 / year"
     )
     expected.loc[{"area (ISO3)": "BOL", "time": "2002"}] = 2 * ureg("Gg CO2 / year")
 
